package obelus2;

import java.util.ArrayList;
import java.util.List;

/**
 * Diagram Node.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
public class DNode implements Cloneable {
	private String oId;
	private String parentId;
	// Need children nodes to be ordered. Here id's are contained (get method will look up in dao).
	private List<String> children;
	private String name;
	private String content;
	// TT-REDESIGN Consider the diagram node might refer to a relation/arc between ontology nodes (e.g. as in a compilation of links)
	private String linkedONode;
	
	public DNode(String oId, String name, String content) {
		this.oId = oId;
		this.name = name;
		this.content = content;
		children = new ArrayList<String>();
	}
	
	public String getOId() { return oId; }
	public String getParentId() { return parentId; }
	public String getName() { return name; }
	public String getContent() { return content; }
	public void setName(String name) { this.name = name; }
	public void setContent(String content) { this.content = content; }
	
	public List<? super DNode> getChildren (List<? super DNode> targetList) {
		return ObelusUtilities.getDao().getDNodeChildren(oId, children, targetList);
	}
	public int getChildrenCount() { return children.size(); }
	public DNode getChildAtIndex(int i) {
		String childId = children.get(i);
		return ObelusUtilities.getDao().getDNode(childId);
	}
	public void addChild(DNode child) {
		children.add(child.getOId());
		child.parentId = this.oId;
	}
	

	public ONode getLinkedONode() {
		if (linkedONode == null) {
			return null;
		} else {
			return ObelusUtilities.getDao().getONode(linkedONode);
		}
	}

	public void setLinkedONode(ONode newLinkedONode) {
		if (newLinkedONode == null) {
			this.linkedONode = null;
		} else {
			this.linkedONode = newLinkedONode.getOId();
		}
	}

	public DNode clone() {
		try {
			return (DNode) super.clone();
		} catch (CloneNotSupportedException e){
			ObelusUtilities.handleInconceivable(e);
			throw new Error(e); //operation failed.
		}
	}

	@Override
	public String toString() {
		return oId + "-" + name;
	}

}
