package obelus2;

/**
 * Contains a representation of knowledge based on the ontology nodes.
 * So far it is a hierarchical representation, with diagram nodes spawning from parent nodes in the diagram.
 * Each diagram node might be linked with the ontology nodes.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
// TT-REDESIGN A Diagram and its nodes might be seen as a type or specific application of generic nodes.
// TT-REDESIGN There might be other kind of diagrams (e.g. a Venn Diagram, A periodic table, etc)
public class Diagram implements Cloneable {
	private String oId;
	private String name;
	// TT-REDESIGN Consider having a creation date
	private String rootId;
	
	public Diagram(String oId, String name, String rootId) {
		this.oId = oId;
		this.name = name;
		this.rootId = rootId;
	}
	
	public String getOId() { return oId; }
	public String getName() { return name; }
	public void setName(String name) { this.name = name; }
	
	public DNode getRoot() {
		return ObelusUtilities.getDao().getDNode(rootId);
	}
	
	public Diagram clone() {
		try {
			return (Diagram) super.clone();
			// linked elements such as root DNode (and rest of tree) would be awkward to clone.
		} catch (CloneNotSupportedException e){
			ObelusUtilities.handleInconceivable(e);
			throw new Error(e); //operation failed.
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof Diagram)) {
			return false;
		}
		Diagram dia2 = (Diagram)obj;
		return oId.equals(dia2.oId) && name.equals(dia2.name);
		// So far rootId is not expected to change between copies. Seems futile to check further.
	}

	@Override
	public String toString() {
		return oId + "-" + name;
	}
	
	
}
