package obelus2;

/**
 * Ontology Arc connects ontology nodes.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
public class OArc implements Cloneable {
	private String oId;
	private String arcType;
	// Fake string type, getters/setters do actual ONode
	private String nodeSource;
	private String nodeTarget;
	
	public OArc() {}

	public OArc(String oId, String arcType, ONode nodeSrc, ONode nodeDst) { 
		super();
		this.oId = oId;
		this.arcType = arcType;
		setNodeSource(nodeSrc);
		setNodeTarget(nodeDst);
	}
	
	public String getOId() {return oId;}
	public void setOId(String oId) {this.oId = oId;}

	public String getArcType() {return arcType;}
	public void setArcType(String arcType) {this.arcType = arcType;}

	public ONode getNodeSource() {
		if (nodeSource == null) {
			return null;
		} else {
			return ObelusUtilities.getDao().getONode(nodeSource);
		}
	}

	public void setNodeSource(ONode newLinkedONode) {
		if (newLinkedONode == null) {
			this.nodeSource = null;
		} else {
			this.nodeSource = newLinkedONode.getOId();
		}
	}


	public ONode getNodeTarget() {
		if (nodeTarget == null) {
			return null;
		} else {
			return ObelusUtilities.getDao().getONode(nodeTarget);
		}
	}

	public void setNodeTarget(ONode newLinkedONode) {
		if (newLinkedONode == null) {
			this.nodeTarget = null;
		} else {
			this.nodeTarget = newLinkedONode.getOId();
		}
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) { return true; }
		if (obj == null || !(obj instanceof OArc)) { return false; }
		OArc arc2 = (OArc)obj;
		if (! oId.equals(arc2.oId)) { return false; }
		if ((arcType == null) ? arc2.arcType != null : ! arcType.equals(arc2.arcType)) { return false; }
		if ((nodeSource == null) ? arc2.nodeSource != null : ! nodeSource.equals(arc2.nodeSource)) { return false; }
		if ((nodeTarget == null) ? arc2.nodeTarget != null : ! nodeTarget.equals(arc2.nodeTarget)) { return false; }
		return true;
	}

	public OArc clone() {
		try {
			return (OArc) super.clone();
		} catch (CloneNotSupportedException e){
			ObelusUtilities.handleInconceivable(e);
			throw new Error(e); //operation failed.
		}
	}

	@Override
	public String toString() {
		return String.format("(%s)%s->%s", arcType, nodeSource, nodeTarget);
	}


}
