package obelus2;

import java.util.ArrayList;
import java.util.List;

/**
 * Ontology Node.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
public class ONode implements Comparable<ONode>, Cloneable {
	private String oId;
	private String nodeType;
	private String name;
	private String content;
	// Not the arcs themselves but their ids
	// Not all the related arcs in the universe, but just those registered in the node.
	private List<String> arcs = new ArrayList<String>();
		
	public ONode() {
		super();
		this.oId = "unassigned";
		this.nodeType = "unassigned";
		this.name = "";
		this.content = "";
	}
	
	public ONode(String oId, String nodeType, String name, String content) {
		super();
		this.oId = oId;
		this.nodeType = nodeType;
		this.name = name;
		this.content = content;
	}
	
	public String getOId() {
		return oId;
	}
	public void setOId(String oId) {
		this.oId = oId;
	}
	public String getNodeType() {
		return nodeType;
	}
	public void setNodeType(String nodeType) {
		this.nodeType = nodeType;
	}
	public String getName() {
		return name;
	}
	public void setName(String alias) {
		this.name = alias;
	}
	public String getContent() {
		return content;
	}
	public void setContent(String content) {
		this.content = content;
	}
	public List<String> getArcs() { // TT-LOW Redesign accesor methods
		return arcs;
	}
	public void setArcs(List<String> arcs) {
		this.arcs = arcs;
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((oId == null) ? 0 : oId.hashCode());
		return result;
	}
	
	// Equals might be used to tell if an element copy has been modified
	@Override
	public boolean equals(Object obj) {
		if (this == obj) { return true; }
		if (obj == null || ! (obj instanceof ONode)) { return false; }
		ONode n2 = (ONode) obj;
		if (! oId.equals(n2.oId)) { return false; }
		if (! nodeType.equals(n2.nodeType)) { return false; }
		if (! name.equals(n2.name)) { return false; }
		if (! content.equals(n2.content)) { return false; }
		if (! arcs.equals(n2.arcs)) { return false; }
		return true;
	}

	@Override
	public int compareTo(ONode n) {
		int diff = this.name.compareTo(n.name);
		if (diff == 0) diff = this.oId.compareTo(n.oId);
		return diff;
	}
	@Override
	public String toString() {
		return name + "(" + nodeType + ")@" + oId;
	}
		
	public ONode clone() {
		try {
			return (ONode) super.clone();
		} catch (CloneNotSupportedException e){
			ObelusUtilities.handleInconceivable(e);
			throw new Error(e); //operation failed.
		}
	}

}
