package obelus2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;


public class ObelusDao {
	//List<ONode> getAllNodes(List<ONode> intoList);
	//ONode getNodeForId(String oId);
	//void insertNode(ONode newNode);
	//void updateNode(ONode updated);
	//void removeNode(ONode erased);

//	List<Node> getNodesForKeyword(String keyword, List<Node> intoList);

	//List<OArc> getAllArcs(List<OArc> intoList);
	//OArc getArcForId(String oId);

	private ArrayList<ONode> oNodes = new ArrayList<ONode>();
	private HashMap<String, ONode> htONodeById = new HashMap<String, ONode>();
	private ArrayList<OArc> oArcs = new ArrayList<OArc>();
	private HashMap<String, OArc> htOArcById = new HashMap<String, OArc>();
	private ArrayList<Diagram> diagrams = new ArrayList<Diagram>();
	private HashMap<String, Diagram> htDiagramById = new HashMap<String, Diagram>();
	private ArrayList<DNode> dNodes = new ArrayList<DNode>();
	private HashMap<String, DNode> htDNodeById = new HashMap<String, DNode>();
	
	//
	// Diagrams
	//
	/**
	 * Create a new diagram object (not yet persistent)
	 */
	public Diagram newDiagram() {
		DNode rootNode = newDNode(null);
		String rootId = rootNode.getOId();
		Diagram di = new Diagram(rootId, "new Diagram", rootId);
		diagrams.add(di);
		htDiagramById.put(rootId, di);
		
		return di;
	}

	public Diagram getDiagram(String oId) {
		return htDiagramById.get(oId);
	}
	
	public List<? super Diagram> searchDiagram(String criteria, List<? super Diagram> intoList) {
		// TT-REDESIGN More sophisticated indexed search
		if (criteria.equals("*")) {
			// return all
			intoList.addAll(diagrams);
		} else {
			// traverse diagrams. pick up those containing criteria
			String criteriaLower = criteria.toLowerCase();
			for (Diagram d : diagrams) {
				if (d.getOId().toLowerCase().contains(criteriaLower) ||
						d.getName().toLowerCase().contains(criteriaLower)) {
					intoList.add(d);
				}
			}
		}
		return intoList;
	}

	public void updateDiagram(Diagram updatedDiagram) {
		// Get and replace previous
		String id = updatedDiagram.getOId();
		Diagram oldDiagram = htDiagramById.get(id);
		int i = diagrams.indexOf(oldDiagram);
		// i >= 0 or else will throw exception
		diagrams.set(i, updatedDiagram);
		htDiagramById.put(id, updatedDiagram);
	}
	
	public void deleteDiagram(Diagram deleted) {
		String oId = deleted.getOId();
		diagrams.remove(deleted);
		htDiagramById.remove(oId);
	}
	
	//
	// Diagram Nodes
	//
	public DNode newDNode(DNode parentNode) {
		String newId = ObelusUtilities.newId();
		DNode node = new DNode(newId, newId, "@DESCRIPTION@");
		dNodes.add(node);
		htDNodeById.put(newId, node);
		if (parentNode != null) {
			parentNode.addChild(node);
			updateDNode(parentNode);
		}
		return node;
	}
	
	public DNode getDNode(String oId) {
		return htDNodeById.get(oId);
	}
	
	public List<? super DNode> getDNodeChildren(String parentId, List<String> childrenIdList, List<? super DNode> targetList) {
		for (String forId : childrenIdList) {
			targetList.add(htDNodeById.get(forId)); // as in getDNode(id)
		}
		return targetList;
	}
	
	public void updateDNode(DNode updatedNode) {
		// Get and replace previous
		String id = updatedNode.getOId();
		DNode oldDiagram = htDNodeById.get(id);
		int i = dNodes.indexOf(oldDiagram);
		// i >= 0 or else will throw exception
		dNodes.set(i, updatedNode);
		htDNodeById.put(id, updatedNode);
	}
	
	public void deleteDNode(DNode deleted) {
		String oId = deleted.getOId();
		dNodes.remove(deleted);
		htDNodeById.remove(oId);
	}

	
	//
	// Ontology Nodes
	//
	public ONode newONode() {
		String newId = ObelusUtilities.newId();
		ONode node = new ONode(newId, "undefined", newId, "");
		oNodes.add(node);
		htONodeById.put(newId, node);
		return node;
	}

	public ONode getONode(String oId) {
		return htONodeById.get(oId);
	}

	public List<? super ONode> searchONode(String criteria, List<? super ONode> intoList) {
		if (criteria.equals("*")) {
			// return all
			intoList.addAll(oNodes);
		} else {
			// traverse diagrams. pick up those containing criteria
			String criteriaLower = criteria.toLowerCase();
			for (ONode n: oNodes) {
				if (n.getOId().toLowerCase().contains(criteriaLower) ||
						n.getName().toLowerCase().contains(criteriaLower) ||
						n.getContent().toLowerCase().contains(criteriaLower)) {
					intoList.add(n);
				}
			}
		}
		return intoList;
	}

	public void updateONode(ONode updatedNode) {
		// Get and replace previous
		String id = updatedNode.getOId();
		ONode oldDiagram = htONodeById.get(id);
		int i = oNodes.indexOf(oldDiagram);
		// i >= 0 or else will throw exception
		oNodes.set(i, updatedNode);
		htONodeById.put(id, updatedNode);
	}
	
	public void deleteONode(ONode deleted) {
		String oId = deleted.getOId();
		oNodes.remove(deleted);
		htONodeById.remove(oId);
	}

	
	//
	// Ontology Arcs
	//
	public OArc newOArc(ONode nodeSrc, ONode nodeT, String arcType) {
		String newId = ObelusUtilities.newId();
		OArc newArc= new OArc(newId, arcType, nodeSrc, nodeT);
		oArcs.add(newArc);
		htOArcById.put(newId, newArc);
		return newArc;
	}

	public OArc getOArc(String oId) {
		return htOArcById.get(oId);
	}

	public void updateOArc(OArc updatedArc) {
		// Get and replace previous
		String id = updatedArc.getOId();
		OArc oldArc = htOArcById.get(id);
		int i = oArcs.indexOf(oldArc);
		// i >= 0 or else will throw exception
		oArcs.set(i, updatedArc);
		htOArcById.put(id, updatedArc);
	}
	
	public void deleteOArc(OArc deleted) {
		String oId = deleted.getOId();
		oArcs.remove(deleted);
		htOArcById.remove(oId);
	}

	
}
