package obelus2.swing;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import obelus2.DNode;
import obelus2.ONode;
import obelus2.ObelusUtilities;

/**
 * Panel for editing a DNode.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
@SuppressWarnings("serial")
public class DNodePanel extends PagePanel {
// TT-REDESIGN There should be a way to reset (assign null) the ONode reference
	protected DNode node;

	protected JTextField idField; 
	protected JTextField nameField; 
	protected JTextField contentField; 
	protected JTextField oNodeField;
	protected JButton oNodeButton;

	protected InnerListener listener;
	

	public DNodePanel() {
		super();
		setLayout(new GridBagLayout());
		
		listener = new InnerListener();

		int layoutRow = 0;
		
		idField = new JTextField();
		idField.addFocusListener(listener);
		idField.setEditable(false);
		ObelusUtilities.addGridbagLabeledProperty(this, "Id", idField, layoutRow++);

		nameField = new JTextField();
		nameField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Name", nameField, layoutRow++);

		contentField = new JTextField();
		contentField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Content", contentField, layoutRow++);

		JPanel linkedNodeContainer = new JPanel(new BorderLayout());
		{
			oNodeField = new JTextField();
			oNodeField.addFocusListener(listener);
			oNodeField.setEditable(false);
			linkedNodeContainer.add(oNodeField, BorderLayout.CENTER);
			
			oNodeButton = new JButton("Search");
			oNodeButton.addActionListener(listener);
			linkedNodeContainer.add(oNodeButton, BorderLayout.EAST);
		}
		
		ObelusUtilities.addGridbagLabeledProperty(this, "Linked Node", linkedNodeContainer, layoutRow++);

	}
	
	/**
	 * Sets the node edited by the panel.
	 * Please use {@link #setEditedContent(Cloneable)} in order to support commit/cancel buttons.
	 * @param newNode Shouldn't be null
	 */
	public void setNode(DNode newNode) {
		this.node = newNode;
		assert node != null;
		idField.setText(node.getOId());
		nameField.setText(node.getName());
		contentField.setText(node.getContent());
		ONode linkedNode = node.getLinkedONode();
		if (linkedNode == null) {
			oNodeField.setText("");
		} else {
			oNodeField.setText(linkedNode.getName());
		}
	}
	
	@Override
	public void setEditedContent(Cloneable content) {
		assert content instanceof DNode;
		super.setEditedContent(content);
		DNode node2 = (DNode)getPageProperty(PROPERTY_CONTENT_EDITED);
		setNode(node2);
		
	}

	/**
	 * Set linked node when returning from nested interaction.
	 * @param linkedNode
	 */
	protected void setLinkedNode(ONode linkedNode) {
		oNodeField.setText(linkedNode.getName());
		node.setLinkedONode(linkedNode);
		checkEdited();
	}

	// All the listener behavior in same class
	// Handle text fields editing
	// Handle button to search linked ontology node
	protected class InnerListener implements ActionListener, FocusListener {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == oNodeButton) {
				SwingMain.onDNodePanelSearchLinked(DNodePanel.this);
			}else if (source == nameField) {
				node.setName(nameField.getText());
				checkEdited();
			}else if (source == contentField) {
				node.setContent(contentField.getText());
				checkEdited();
			}
		}

		@Override
		public void focusGained(FocusEvent e) {
			// nothing to do?
			
		}

		@Override
		public void focusLost(FocusEvent e) {
			Object source = e.getSource();
			if (source == nameField) {
				node.setName(nameField.getText());
				checkEdited();
			} else if (source == contentField) {
				node.setContent(contentField.getText());
				checkEdited();
			}

		}

		
	}
}
