package obelus2.swing;

import java.util.ArrayList;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import obelus2.DNode;

public class DiagramTreeModel implements TreeModel {

	private DNode rootNode;
	
	public DiagramTreeModel(DNode rootNode) {
		this.rootNode = rootNode;
	}
	
	@Override
	public Object getRoot() {
		return rootNode;
	}

	@Override
	public Object getChild(Object parent, int index) {
		DNode node = (DNode)parent;
		return node.getChildAtIndex(index);
	}

	@Override
	public int getChildCount(Object parent) {
		DNode node = (DNode)parent;
		return node.getChildrenCount();
	}

	@Override
	public boolean isLeaf(Object node) {
		return getChildCount(node) == 0;
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) {
		DNode node = (DNode)parent;
		ArrayList<DNode> children = new ArrayList<DNode>(node.getChildrenCount());
		node.getChildren(children);
		return children.indexOf(child);
	}

	@Override
	public void valueForPathChanged(TreePath path, Object newValue) {
		assert newValue != null;
		// Still hassling with no upgrade problem. maybe path should have the newValue in it.
//		// node instance at path might be different. replace it
//		TreePath parentPath = path.getParentPath();
//		if (parentPath == null) { // root node
//			this.rootNode = (DNode)newValue;
//		} else {
//			DNode oldValue = (DNode)path.getLastPathComponent();
//			List<? super DNode> siblings = ((DNode)parentPath.getLastPathComponent()).getChildren(new ArrayList<DNode>()); // TT-LOW Find a proper generics approach
//			int pos = siblings.indexOf(oldValue);
//			// TODO Update is a mess, might be making a redundant update in wrong instance
//			//if (pos >= 0) { ((DNode)parentPath.getLastPathComponent()).setChildAt(pos, (DNode)newValue); }
//		}
		// The item was edited, or changed for some reason. Assume structure still the same
		
		if (path.getParentPath() == null) { this.rootNode = (DNode)newValue; }
		TreeModelEvent evt = new TreeModelEvent(this, path);
		for (TreeModelListener l : treeModelListeners) {
//			l.treeNodesChanged(evt);
// Does not update saying just value change
			l.treeStructureChanged(evt);
		}
	}
	
	protected void nodeStructureChanged(TreePath path) {
		TreeModelEvent evt = new TreeModelEvent(this, path);
		for (TreeModelListener l : treeModelListeners) {
			l.treeStructureChanged(evt);
		}
	}
	
	private ArrayList<TreeModelListener> treeModelListeners = new ArrayList<TreeModelListener>(1);
	@Override
	public void addTreeModelListener(TreeModelListener l) {
		treeModelListeners.add(l);
	}

	@Override
	public void removeTreeModelListener(TreeModelListener l) {
		treeModelListeners.remove(l);
	}
	
}
