package obelus2.swing;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import obelus2.OArc;
import obelus2.ONode;
import obelus2.ObelusUtilities;

/**
 * Panel for editing a DNode.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
@SuppressWarnings("serial")
public class OArcPanel extends PagePanel {
// TT-REDESIGN [NavigateToNode]
// Currently the button opens the search interface, but there should be a way to display the node panel for the linked node.
	protected OArc arc;

	protected JTextField idField; 
	protected JTextField typeField; 
	protected JTextField sourceField; 
	protected JTextField targetField;
	protected JButton targetButton;

	protected InnerListener listener;
	

	public OArcPanel() {
		super();
		setLayout(new GridBagLayout());
		
		listener = new InnerListener();

		int layoutRow = 0;
		
		idField = new JTextField();
		idField.addFocusListener(listener);
		idField.setEditable(false);
		ObelusUtilities.addGridbagLabeledProperty(this, "Id", idField, layoutRow++);

		typeField = new JTextField();
		typeField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Type", typeField, layoutRow++);

		sourceField = new JTextField();
		sourceField.addFocusListener(listener);
		idField.setEditable(false);
		ObelusUtilities.addGridbagLabeledProperty(this, "Source", sourceField, layoutRow++);

		JPanel targetContainer = new JPanel(new BorderLayout());
		{
			targetField = new JTextField();
			targetField.addFocusListener(listener);
			targetField.setEditable(false);
			targetContainer.add(targetField, BorderLayout.CENTER);
			
			targetButton = new JButton("Search");
			targetButton.addActionListener(listener);
			targetContainer.add(targetButton, BorderLayout.EAST);
		}
		
		ObelusUtilities.addGridbagLabeledProperty(this, "Target", targetContainer, layoutRow++);

	}
	
	/**
	 * Sets the arc edited by the panel.
	 * Please use {@link #setEditedContent(Cloneable)} in order to support commit/cancel buttons.
	 * @param newArc Shouldn't be null
	 */
	public void setArc(OArc newArc) {
		this.arc = newArc;
		assert arc != null;
		idField.setText(arc.getOId());
		typeField.setText(arc.getArcType());
		ONode node = arc.getNodeSource();
		sourceField.setText(node == null ? "(unassigned)" : node.getName());
		node = arc.getNodeTarget();
		targetField.setText(node == null ? "(unassigned)" : node.getName());
	}
	
	@Override
	public void setEditedContent(Cloneable content) {
		assert content instanceof OArc;
		super.setEditedContent(content);
		OArc arc2 = (OArc)getPageProperty(PROPERTY_CONTENT_EDITED);
		setArc(arc2);
		
	}
	
	public void setTargetNode(ONode newTarget) {
		arc.setNodeTarget(newTarget);
		targetField.setText(newTarget == null ? "(unassigned)" : newTarget.getName());
		checkEdited();
	}


	// All the listener behavior in same class
	// Handle text fields editing
	// Handle button to search linked ontology node
	protected class InnerListener implements ActionListener, FocusListener {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == targetButton) {
				SwingMain.onOArcPanelSearchTarget(OArcPanel.this);
			} else {
				assert source == typeField;
				arc.setArcType(typeField.getText());
				checkEdited();
			}
		}

		@Override
		public void focusGained(FocusEvent e) {
			// nothing to do?
			
		}

		@Override
		public void focusLost(FocusEvent e) {
			Object source = e.getSource();
			if (source == typeField) {
				arc.setArcType(typeField.getText());
				checkEdited();
			}

		}

		
	}
}
