package obelus2.swing;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;

import obelus2.OArc;
import obelus2.ONode;
import obelus2.ObelusUtilities;

/**
 * Panel for editing an Ontology Node.
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
@SuppressWarnings("serial")
public class ONodePanel extends PagePanel {

	protected ONode node;

	protected JTextField idField;
	protected JTextField typeField;
	protected JTextField nameField; 
	protected JTextField contentField;
	protected JList<OArc> arcsList;	
	
	protected JPopupMenu arcsPopup;
	protected JMenuItem addArcMenu;
	protected JMenuItem editArcMenu;

	protected InnerListener listener;
	
	

	public ONodePanel() {
		super();
		setLayout(new GridBagLayout());
		
		listener = new InnerListener();

		int layoutRow = 0;
		
		idField = new JTextField();
		idField.addFocusListener(listener);
		idField.setEditable(false);
		ObelusUtilities.addGridbagLabeledProperty(this, "Id", idField, layoutRow++);

		typeField = new JTextField();
		typeField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Type", typeField, layoutRow++);

		nameField = new JTextField();
		nameField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Name", nameField, layoutRow++);

		contentField = new JTextField();
		contentField.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Content", contentField, layoutRow++);

		arcsList = new JList<OArc>();
		arcsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
//		arcsList.getSelectionModel().addListSelectionListener(listener);
		arcsList.addMouseListener(listener);
		arcsList.setCellRenderer(new ArcCellRenderer());
		JScrollPane listScroll = new JScrollPane(arcsList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listScroll.setBorder(BorderFactory.createTitledBorder("Arcs"));
		add(listScroll, new GridBagConstraints(0, layoutRow++, 2, 1, 1.0, 1.0,
				GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
				new Insets(2,2,2,2), 0, 0));
		
		// Pop-up menu 
		arcsPopup = new JPopupMenu();

		addArcMenu = new JMenuItem("Add arc");
		addArcMenu.setActionCommand("ADD-ARC");
		addArcMenu.addActionListener(listener);
		arcsPopup.add(addArcMenu);
		
		editArcMenu = new JMenuItem("Edit");
		editArcMenu.setActionCommand("EDIT-ARC");
		editArcMenu.addActionListener(listener);
		arcsPopup.add(editArcMenu);

	}
	
	/**
	 * Sets the node edited by the panel.
	 * Please use {@link #setEditedContent(Cloneable)} in order to support commit/cancel buttons.
	 * @param newNode Shouldn't be null
	 */
	public void setNode(ONode newNode) {
		this.node = newNode;
		assert node != null;
		idField.setText(node.getOId());
		typeField.setText(node.getNodeType());
		nameField.setText(node.getName());
		contentField.setText(node.getContent());
		
		DefaultListModel<OArc> newModel = new DefaultListModel<OArc>();
		for (String arcId : newNode.getArcs()) {
			newModel.addElement(ObelusUtilities.getDao().getOArc(arcId));
		}
		arcsList.setModel(newModel);
	}
	
	@Override
	public void setEditedContent(Cloneable content) {
		assert content instanceof ONode;
		super.setEditedContent(content);
		ONode node2 = (ONode)getPageProperty(PROPERTY_CONTENT_EDITED);
		setNode(node2);
		
	}

	protected void showArcsPopup (MouseEvent e) {
		// Check items that should be disabled
		if (arcsList.getSelectedIndices().length == 1) {
//			addArcMenu.setEnabled(true);
			editArcMenu.setEnabled(true);
		} else {
//			addArcMenu.setEnabled(true);
			editArcMenu.setEnabled(false);
		}
		arcsPopup.show(e.getComponent(), e.getX(), e.getY());
	}
	

	// All the listener behavior in same class
	// Handle text fields editing
	// Handle menus
	protected class InnerListener implements ActionListener, FocusListener, MouseListener {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == typeField) {
				node.setNodeType(typeField.getText());
				checkEdited();
			} else if (source == nameField) {
				node.setName(nameField.getText());
				checkEdited();
			} else if (source == contentField) {
				node.setContent(contentField.getText());
				checkEdited();
			} else if (source == addArcMenu) {
				SwingMain.onONodePanelAddArc(ONodePanel.this);
			} else if (source == editArcMenu) {
				SwingMain.onONodePanelEditArc(ONodePanel.this);
			} 
		}

		@Override
		public void focusGained(FocusEvent e) {
			// nothing to do?
			
		}

		@Override
		public void focusLost(FocusEvent e) {
			Object source = e.getSource();
			if (source == typeField) {
				node.setNodeType(typeField.getText());
				checkEdited();
			} else if (source == nameField) {
				node.setName(nameField.getText());
				checkEdited();
			} else if (source == contentField) {
				node.setContent(contentField.getText());
				checkEdited();
			}

		}

		//
		// Mouse Events
		@Override
		public void mouseClicked(MouseEvent e) {
			// nothing to do so far
		}
		
		@Override
		public void mousePressed(MouseEvent e) {
			if (e.getComponent() == arcsList) {
				if (SwingUtilities.isRightMouseButton(e)) {
//					int rowSelected = arcsList.getRowForLocation(e.getX(), e.getY());
					int rowSelected = arcsList.locationToIndex(e.getPoint());
//					arcsList.setSelectionRow(rowSelected);
					arcsList.setSelectedIndex(rowSelected);
				}
				if (e.isPopupTrigger()) {
					showArcsPopup(e);
				}
			}
		}
		@Override
		public void mouseReleased(MouseEvent e) {
			if (e.getComponent() == arcsList) {
				if (e.isPopupTrigger()) {
					showArcsPopup(e);
				}
			}
		}
		@Override
		public void mouseEntered(MouseEvent e) {
			// nothing to do
		}

		@Override
		public void mouseExited(MouseEvent e) {
			// nothing to do
			
		}
		

		
	}
	
	protected class ArcCellRenderer extends DefaultListCellRenderer {

		@Override
		public Component getListCellRendererComponent(JList<?> list,Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected,cellHasFocus);
			if (value instanceof OArc) {
				OArc arc = (OArc)value;
				ONode arcSource = arc.getNodeSource();
				ONode arcTarget = arc.getNodeTarget();
				setText(String.format("(%s)%s->%s ",
						arc.getArcType(),
						arcSource == null ? "(undefined)" : arcSource.getName(),
						arcTarget == null ? "(undefined)" : arcTarget.getName()));
			}
			return this;
		}

	}
}
