package obelus2.swing;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import obelus2.ObelusUtilities;

/**
 * Panel for searching/selecting items (either Diagrams or Ontology nodes).
 * The composition of either item searching would be very similar, so the same class shall be used, though
 * #1 specific renderer should be supplied for the list item
 * #2 The controller should be able to tell apart the type when invoked back (e.g. through the context).
 * @author Javier Aranda (javier-aranda.com)
 * CC SA BY
 */
@SuppressWarnings("serial")
public class SearchPanel extends PagePanel {
	
	protected String criteria;
	protected List<Object> searchResult;
	protected Object selectedItem;
	
	protected InnerListener listener;
	
	protected JTextField textCriteria;
	protected JList<Object> list;
	protected ListCellRenderer<Object> listRenderer;
	
//	protected JButton buttonSelect;
//	protected JButton buttonCreateNew;
//	protected JButton buttonCancel;
	
	public SearchPanel () {
		listener = new InnerListener();
		
		setLayout(new GridBagLayout());
		textCriteria = new JTextField();
		textCriteria.addActionListener(listener);
		// Focus lost should trigger search?
		// textCriteria.addFocusListener(listener);
		ObelusUtilities.addGridbagLabeledProperty(this, "Criteria", textCriteria, 0);

		list = new JList<Object>(new String[]{"Enter criteria an press Enter to search items"});
		list.setEnabled(false);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.getSelectionModel().addListSelectionListener(listener);
		JScrollPane listScroll = new JScrollPane(list, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		add(listScroll, new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0,
				GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
				new Insets(2,2,2,2), 0, 0));
		
		
	}

	public String getCriteria() {
		return criteria;
	}
	public void setCriteria(String criteria) {
		this.criteria = criteria;
		textCriteria.setText(criteria);
		if (! criteria.trim().isEmpty()) {
			SwingMain.onSearchPanelCriteria(this);
		}
	}

	public void setSearchResult(List<Object> searchResult) {
		this.searchResult = searchResult;
		if (searchResult.size() > 0) {
			list.setCellRenderer(listRenderer);
			DefaultListModel<Object> model = new DefaultListModel<>();
			model.removeAllElements();
			for (Object element : searchResult) {
				model.addElement(element);
			}
			list.setModel(model);
			list.setEnabled(true);
		} else {
			// Nothing found. Render text and disable selection
			list.setCellRenderer(new DefaultListCellRenderer());
			DefaultListModel<Object> model = new DefaultListModel<>();
			model.removeAllElements();
			model.addElement("No results found for criteria");
			list.setModel(model);
			list.setEnabled(false);
		}
	}

	public Object getSelectedItem() {
		return selectedItem;
	}

	protected void setSelectedItem(Object newSelected) {
		if (newSelected != selectedItem) {
			firePropertyChange("selectedItem", selectedItem, newSelected);
			selectedItem = newSelected;
		}
	}

	public void setListRenderer(ListCellRenderer<Object> customRenderer) {
		this.listRenderer = customRenderer;
	}
	
	protected class InnerListener implements ActionListener, ListSelectionListener {
		// ActionListener
		@Override
		public void actionPerformed(ActionEvent e) {
			Object src = e.getSource();
			if (src == textCriteria) {
				criteria = textCriteria.getText();
				SwingMain.onSearchPanelCriteria(SearchPanel.this);
			}
//			else if (src == buttonSelect) {
//				selectedItem = list.getSelectedValue();
//				SwingController.instance().onSearchPanelSelect(SearchPanel.this);
//			} else if (src == buttonCreateNew) {
//				SwingController.instance().onSearchPanelCreate(SearchPanel.this);
//			} else if (src == buttonCancel) {
//				SwingController.instance().onSearchPanelCancel(SearchPanel.this);
//			}
		}

		// ListSelectionListener
		// - Enable/disable related buttons
		@Override
		public void valueChanged(ListSelectionEvent e) {
			assert e.getSource() == list;
			setSelectedItem(list.getSelectedValue());
		}
		
	}
}
